package es.uv.garcosda.controllers;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import es.uv.garcosda.domain.Post;
import es.uv.garcosda.domain.PostDTO;
import es.uv.garcosda.services.PostService;
import jakarta.validation.ConstraintViolation;
import jakarta.validation.ConstraintViolationException;
import jakarta.validation.Valid;


@RestController
@RequestMapping("/api/v1")
public class BlogController {
	private final static Logger LOGGER = LoggerFactory.getLogger(BlogController.class);
	
	@Autowired 
	private PostService ps;
	
	@GetMapping("posts")
	public ResponseEntity<List<Post>> findAll() {
		return new ResponseEntity<List<Post>>(this.ps.findAll(), HttpStatus.OK);
	}
	
	@GetMapping("posts/{id}")
	public ResponseEntity<Post> findById(@PathVariable("id") String id) {
		Post p = ps.findById(id);
		if(p.getId() != null) {
			return new ResponseEntity<Post>(p, HttpStatus.OK);
		}
		else {
			return new ResponseEntity<Post>(new Post(), HttpStatus.NOT_FOUND);
		}
	}
	
	@PostMapping("posts")
	public ResponseEntity<Post> create(@Valid @RequestBody PostDTO dto){
		Post p = ps.add(dto);
		if(p.getId() != null) {
			return new ResponseEntity<Post>(p, HttpStatus.OK);
		}
		else {
			return new ResponseEntity<Post>(new Post(), HttpStatus.NOT_FOUND);
		}
	}
	
	@DeleteMapping("posts")
	public ResponseEntity<String> delete(@RequestBody Post p){
		if(ps.delete(p)) {
			return new ResponseEntity<String>("", HttpStatus.OK);
		}
		else {
			return new ResponseEntity<String>("", HttpStatus.NOT_FOUND);
		}
	}
	
	@DeleteMapping("posts/{id}")
	public ResponseEntity<String> deleteById(@PathVariable String id){
		if(ps.deleteById(id)) {
			return new ResponseEntity<String>("", HttpStatus.OK);
		}
		else {
			return new ResponseEntity<String>("", HttpStatus.NOT_FOUND);
		}
	}
	
	@PutMapping("posts")
	public ResponseEntity<Post> update(@RequestBody Post dto){
		Post p = ps.update(dto);
		if(p.getId() != null) {
			return new ResponseEntity<Post>(p, HttpStatus.OK);
		}
		else {
			return new ResponseEntity<Post>(new Post(), HttpStatus.NOT_FOUND);
		}
	}
	
	@ExceptionHandler(ConstraintViolationException.class)
	public ResponseEntity<List<String>> validationError(ConstraintViolationException ex) {
		List<String> msgs = new ArrayList<String>();
		for(ConstraintViolation<?> v : ex.getConstraintViolations()) {
			msgs.add(v.getPropertyPath().toString() + ": " + v.getMessage()); 
		}
	    return new ResponseEntity<List<String>>(msgs, HttpStatus.BAD_REQUEST);
	}
	
	@ExceptionHandler(MethodArgumentNotValidException.class)
	public ResponseEntity<List<String>> validationError(MethodArgumentNotValidException ex) {
		List<String> msgs = new ArrayList<String>();
		List<FieldError> errors = ex.getFieldErrors();
		for(FieldError e : errors) {
			msgs.add(e.getField() + ": " + e.getDefaultMessage()); 
		}
	    return new ResponseEntity<List<String>>(msgs, HttpStatus.BAD_REQUEST);
	}
	
}
